@@ -360,6 +360,10 @@
  #define    _FEATURE_081_PPA_IN_ORDER_FACILITY
 #endif
 
+#if defined( FEATURE_084_MISC_INSTR_EXT_FACILITY_4 )
+ #define    _FEATURE_084_MISC_INSTR_EXT_FACILITY_4
+#endif
+
 #if defined( FEATURE_129_ZVECTOR_FACILITY )
  #define    _FEATURE_129_ZVECTOR_FACILITY
 #endif
