@@ -895,7 +895,7 @@ int rc = 0;
 #endif
         if (regs->arch_mode == ARCH_370_IDX)
         {
-            itimer = INT_TIMER(regs);
+            itimer = INTERVAL_TIMER(regs);
         /* The interval timer counts 76800 per second, or one every
            13.0208 microseconds. */
             MSGBUF(itimer_formatted,"%02u:%02u:%02u.%06u",
