@@ -250,7 +250,11 @@
 #define STFL_082_IBM_INTERNAL         82    /* IBM internal use          */
 
 #define STFL_083_UNDEFINED            83    /* Undefined                 */
-#define STFL_084_UNDEFINED            84    /* Undefined                 */
+
+#define STFL_084_MISC_INSTR_EXT_4     84    /* Miscellaneous-instruction-
+                                               extensions facility 4 is
+                                               installed.                */
+
 #define STFL_085_UNDEFINED            85    /* Undefined                 */
 #define STFL_086_UNDEFINED            86    /* Undefined                 */
 #define STFL_087_UNDEFINED            87    /* Undefined                 */
@@ -461,7 +465,7 @@
                                                134, 152 and 192 are also
                                                one.                      */
 
-#define STFL_200_IBM_INTERNAL        153    /* IBM internal use          */
+#define STFL_200_IBM_INTERNAL        200    /* IBM internal use          */
 
 #define STFL_IBM_LAST_BIT            200    /* Last defined IBM facility */
 
